<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class TipoDeCardapioController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
  protected $connection;
  protected $errorCode = [
		"ORA-02292" => "ERROR_REGISTROFILHO"
	];

    public function __construct($crudService, $parameterBag, $connection, $environment) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
	}
	
	public function saveArvTpCard(DTO\Request $request, DTO\Response $response) {
	    try {

          $row = $request->getRow();

          $this->connection->beginTransaction();

          $sequencial = $this->getSeqArvCard($row) + 1;
          $sequencial = str_pad($sequencial, 10, "0", STR_PAD_LEFT);
          $nrNivel = empty($row['NIVELPAI']) ? 1 : $row['NIVELPAI'] + 1;

          $query = "INSERT INTO ARVTIPOCARD (CDFILCENTPME, NRPASTATCARD, NMPASTATCARD, NRNVPASTCARD, NRPASTCARANT, DTULTATUARVT)
                    VALUES (:CDFILCENTPME, :NRPASTATCARD, :NMPASTATCARD, :NRNVPASTCARD, :NRPASTCARANT, SYSDATE)";
            
          $params = array(
            'CDFILCENTPME' => $row['CDFILCENTPME'],
            'NRPASTATCARD' => $sequencial,
            'NMPASTATCARD' => $row['NMPASTATCARD'],
            'NRNVPASTCARD' => $nrNivel,
            'NRPASTCARANT' => $row['NRPASTAPAI']
          );
            
	        $this->connection->prepare($query)->execute($params);
	        
	        $this->connection->commit();
	        $response->addDataSet(new DataSet("save", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  public function saveTipoCard(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = $request->getRow();
      $rowsToSave = [];
      
      foreach($rows as $row){
        $row['CDTIPOCARD'] = str_pad($row['CDTIPOCARD'], 10, "0", STR_PAD_LEFT);
        $row['DTULTATUTCAR'] = date('d/m/Y');
        $row['DSPASTATCARD'] = null;
		    array_push($rowsToSave, $row);
		  }
      $dataSourceName = 'tipocard';
      $this->callSave($dataSourceName, $rowsToSave);
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveServTpCard(DTO\Request $request, DTO\Response $response) {
    try {

        $this->connection->beginTransaction();

        $rows = (array) $request->getRow();
        $dataSourceName = 'servTipoCard';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveCusto(DTO\Request $request, DTO\Response $response) {
    try {

        $this->connection->beginTransaction();

        $rows = $request->getRow();
        $rowsToSave = [];

        foreach($rows as $row){
          array_push($rowsToSave, $row);
        }
        $dataSourceName = 'custoPadrCard';
        $this->callSave($dataSourceName, $rowsToSave);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveConfigPrat(DTO\Request $request, DTO\Response $response) {
    $this->connection->beginTransaction();
    try {
      $rows = (Array) $request->getRow();
      // $this->deleteConfigPrat($rows);
      $errors = [];
      $nrMaxSeq = $this->getMaxSeqConfPrat()['NRSEQCNFTCAR'];
      $newRows = [];
      foreach ($rows as $row) {
        $row['__is_new'] = true;
        if ($row['NEW'] === 'S') {
          $validacao = $this->validConfigPrat($row);
          if ($validacao !== true) {
            $errors[] = $validacao;
            continue;
          } else {
            $nrMaxSeq += 1;
            $row['NRSEQCNFTCAR'] = $nrMaxSeq;
            $row['NRORDCNFTCAR'] = $nrMaxSeq;
            $newRows[] = $row;
          }
        } else $newRows[] = $row;
      }
      $rows = $newRows;
      $this->deleteConfigPrat($rows);
      $dataSourceName = 'confpadtcard';
      $this->callSave($dataSourceName, $rows);
      $this->connection->commit();
      if (!empty($errors)) {
        $returnError = 'impAdicionarExistentes: '.implode(", ", $errors);
        $response->addDataset(new DataSet('mensagem', array($returnError)));
        return;
      }
      // $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  private function validConfigPrat($row) {
    $nrorg = $this->environment->getNrOrg();
    
    $query = "SELECT CF.CDTIPOPRAT, TC.NMTIPOPRAT FROM CONFPADTCARD CF
            JOIN TIPOPRAT TC ON CF.CDTIPOPRAT = TC.CDTIPOPRAT
           WHERE CF.CDFILCENTPME = :CDFILCENTPME
             AND CF.CDSERVFILI = :CDSERVFILI
             AND CF.CDTIPOPRAT = :CDTIPOPRAT
             AND CF.CDTIPOCARD = :CDTIPOCARD
             AND CF.NRORG = :NRORG";
  
    $params = array(
      'CDFILCENTPME' => $row['CDFILCENTPME'],
      'CDSERVFILI' => $row['CDSERVFILI'],
      'CDTIPOPRAT' => $row['CDTIPOPRAT'],
      'CDTIPOCARD' => $row['CDTIPOCARD'],
      'NRORG' => $nrorg
    );
    $result = $this->connection->fetchAll($query, $params);
    if (count($result) > 0) return $result[0]['NMTIPOPRAT'];
    return true;
  }
  
  private function deleteConfigPrat($rows) {
    $nrorg = $this->environment->getNrOrg();
    
      foreach($rows as $row) {
        $query = "DELETE
                FROM CONFPADTCARD
               WHERE CDFILCENTPME = :CDFILCENTPME
                 AND CDSERVFILI = :CDSERVFILI
                 AND CDTIPOPRAT = :CDTIPOPRAT
                 AND NRSEQCNFTCAR = :NRSEQCNFTCAR
                 AND NRORG = :NRORG";
      
        $params = array(
          'CDFILCENTPME' => $row['CDFILCENTPME'],
          'CDSERVFILI' => $row['CDSERVFILI'],
          'CDTIPOPRAT' => $row['CDTIPOPRAT'],
          'NRSEQCNFTCAR' => $row['NRSEQCNFTCAR'],
          'NRORG' => $nrorg
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
      }
  }

  
  private function getMaxSeqConfPrat() {
    $sql = "SELECT MAX(NRSEQCNFTCAR) NRSEQCNFTCAR
                  FROM CONFPADTCARD";
    
    $resultado = $this->connection->fetchAssoc($sql);
    return $resultado;
  }

  public function saveUnidadeTipoCard(DTO\Request $request, DTO\Response $response) {
    try {
        $this->connection->beginTransaction();
        $rows = (Array) $request->getRow();
        $dataSourceName = 'filitipocardUnidade';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function saveTipoCardUnidade(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $rows = (Array) $request->getRow();
      $dataSourceName = 'filitipocardTipoCard';
      $this->callSave($dataSourceName, $rows);
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function callSave($dataSourceName, $rowsToSave){
    $dataSet = new DataSet($dataSourceName, $rowsToSave);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);    
    $this->crudService->save($dataSourceName, $rowsToSave);
  }
	
	private function getSeqArvCard($row) {
    $query = "SELECT MAX(NRPASTATCARD) MAXNRPASTATCARD FROM ARVTIPOCARD WHERE CDFILCENTPME =:CDFILCENTPME";
    $params = array(
      'CDFILCENTPME' => $row['CDFILCENTPME']
    );
		return $this->connection->fetchColumn($query, $params);
  }
  
  public function deleteTree(DTO\Request $request, DTO\Response $response) {
    try {
        $nrorg = $this->environment->getNrOrg();

        $this->connection->beginTransaction();

        $row = $request->getRow();
        $query = "DELETE FROM ARVTIPOCARD
               WHERE CDFILCENTPME = :CDFILCENTPME
                 AND NRPASTATCARD = :NRPASTATCARD
                 AND NRORG = :NRORG";
      
        $params = array(
          'CDFILCENTPME' => $row['CDFILCENTPME'],
          'NRPASTATCARD' => $row['NRPASTATCARD'],
          'NRORG' => $nrorg
        );
        $stmt = $this->connection->prepare($query)->execute($params);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", []));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $errorMessage = $e->getMessage();
			foreach($this->errorCode as $key => $val){
				if(strpos($errorMessage, $key)) $errorMessage = $val;
			}
			$response->setError(new Error($errorMessage, 500));
    }
  }
  
  public function importaConfigPadraoTpCard(DTO\Request $request, DTO\Response $response) {
    try {
        $parameters = $request->getParameters();
        $rows = $parameters['rowsGridConf'];
        $paramsInsert = $parameters['parametrosDaQueryInsert'];
        
        $this->connection->beginTransaction();


        if ($rows) {
          $query = "DELETE
                    FROM CONFPADTCARD
                   WHERE CDFILCENTPME = :OLD_CDFILCENTPME
                     AND CDTIPOCARD = :OLD_CDTIPOCARD
                     AND CDSERVFILI = :OLD_CDSERVFILI";
            
          foreach($rows as $row){
            $params = array(
              'OLD_CDFILCENTPME' => $row['CDFILCENTPME'],
              'OLD_CDTIPOCARD' => $row['CDTIPOCARD'],
              'OLD_CDSERVFILI' => $row['CDSERVFILI'],
            );
            $this->connection->prepare($query)->execute($params);
          }
        }
        
        $query = "INSERT INTO CONFPADTCARD (CDFILCENTPME, CDTIPOCARD, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR)
                (SELECT CDFILIAL, :P_CDTIPOCARD, CDSERVFILI, NRSEQCNFCARD, CDTIPOPRAT, IDOPCCNFCARD, IDIMPCNFCARD, NRORDCNFCARD
                   FROM CONFCARD
                  WHERE (CDFILIAL = :P_CDFILIAL) 
                    AND (CDSERVFILI = :P_CDSERVFILI)
                )";
            
        $params = array(
          'P_CDTIPOCARD' => $paramsInsert['CDTIPOCARD'],
          'P_CDFILIAL' => $paramsInsert['CDFILIAL'],
          'P_CDSERVFILI' => $paramsInsert['CDSERVFILI'],
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        
        $this->connection->commit();
        $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
    	$this->connection->rollback();
		  $response->setError(new Error($e->getMessage(), 500));
	  }
  }
  
	public function getDataConfigPadraoTpCard(DTO\Request\Row $request, DTO\Response $response) {
		$sql = "SELECT CDFILIAL, CDSERVFILI, NRORG FROM CONFCARD WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI";
		$row = $request->getRow();
		$paramSelect = array(
        "CDSERVFILI" => $row['CDSERVFILI'],
        "CDFILIAL"	 => $row['CDFILIAL']
    );
      
    $dataConfigPadrao = $this->connection->fetchAll($sql, $paramSelect);
    $response->addDataSet(new DataSet("result", $dataConfigPadrao));
	}
	
	public function updateTree(DTO\Request\Row $request, DTO\Response $response) {
	  try {
      $parameters = $request->getParameters();
      $rows = $parameters['rowsEditadas'];
      $nrorg = $this->environment->getNrOrg();
      
      $this->connection->beginTransaction();

      $query = "UPDATE ARVTIPOCARD 
                  SET 
                    NMPASTATCARD = :P_NMPASTATCARD
                  WHERE NRPASTATCARD = :P_NRPASTATCARD
                    AND CDFILCENTPME = :P_CDFILCENTPME
                    AND NRORG        = :P_NRORG";
        
      foreach($rows as $row){
        $params = array(
          'P_NMPASTATCARD' => $row['NMPASTATCARD'],
          'P_NRPASTATCARD' => $row['NRPASTATCARD'],
          'P_CDFILCENTPME' => $row['CDFILCENTPME'],
          'P_NRORG' => $nrorg
        );
        $this->connection->prepare($query)->execute($params);
      }
          
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
	  } catch (\Exception $e) {
	    $this->connection->rollback();
	    $response->setError(new Error($e->getMessage(), 500));
	  }
	}
  
}