import {
  Modal, ModalService, Button, SelectMultiple, Form, DialogService, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam, Loader, Http,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme04001AjustarComensaisEspeciais {
  private modalAjuste!: Modal;
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
    
  public showModal({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.validAndOpenModal();
  }
  
  public async createModal() {
    this.modalAjuste = await ModalService.createFromJson('pme/cardapio/modals/pme04001_ajustar_comensais_especiais', true);
  }
  
  public async validAndOpenModal() {
    if (this.elaboracaoCardapio.getFilterData.NRCARDPME) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(true);
      if (this.utilities.checkOpenModalOverlap()) {
        if (!this.modalAjuste) {
          this.createModal().then(() => {
            this.modalAjuste.show();
          });
        } else {
          this.modalAjuste.show();
        }
      }
    } else {
      this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
    }
  }
  
  public cancelModal() {
    if (this.modalAjuste) {
      this.modalAjuste.destroy();
      this.modalAjuste = this.utilities.modalEmpty;
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
  
  public filterSelectMultiple({ component }: IEventParam<SelectMultiple>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cardapio.filterData.cdfilcentpme,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cardapio.filterData.typeMenu.CDTIPOCARD,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cardapio.filterData.service.CDSERVFILI,
      },
    ];
      
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    const finalDate: IDictionary = Metadata.getInstance('finalDateAjust');
    finalDate.value = null;
  }
  
  public setInitialDate({ component }: IEventParam<IDictionary>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const date = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
    if (component.name === 'initialDatejust') {
      component.value = date;
    } else {
      component.value = date;
    }
  }
  
  public confirmAjuste() {
    const form = Metadata.getInstance<Form>('formAjust');
    if (form.validate()) {
      DialogService.show({
        name: 'excludePeriodoCardapio',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaAjustarComensaisEspPeriodo',
        buttons: [
          {
            name: 'yesButtonAjust',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callAjusteComensaisEspeciais() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public callAjusteComensaisEspeciais() {
    const form: Form = Metadata.getInstance('formAjust');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
      
    if (form.validate()) {
      const initialDate = cardapio.dates.find(
        (date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.initialDateExclude,
      );
      const finalDate = cardapio.dates.find(
        (date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.finalDateExclude,
      );
      if (initialDate !== undefined && finalDate !== undefined) {
        if (!this.bloqDayInPeriod(form.value.initialDateExclude, form.value.finalDateExclude, cardapio)) {
          const row = {
            CDFILCENTPME: this.appController.getCdCentral(),
            ...form.value,
          };
          LoadingService.show();
          Http.post('/procedureAjustComEspPrat', { requestType: 'Row', row }).then((response: IDictionary) => {
            if (response.data.length === 0) {
              this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
              // this.cancelModal();
            }
            // else this.utilities.showAlert('errorResponse', response, 'error');
          }).catch((error: IDictionary) => {
            this.utilities.showAlert('errorResponse', error, 'error');
          }).finally(() => {
            DialogService.hide();
            LoadingService.hide();
          });
        } else {
          this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
          DialogService.hide();
        }
      } else {
        this.utilities.showAlert('outOfPeriod', 'pme::dataForaDoPeriodo', 'error');
        DialogService.hide();
      }
    }
  }
  
  private bloqDayInPeriod(initialDate: string, finalDate: string, cardapio: Cardapio): boolean {
    let hasBloqDay = false;
    cardapio.dates.forEach((date: IDictionary) => {
      if (date.DTCARDIAPME.substring(0, 10) >= initialDate && date.DTCARDIAPME.substring(0, 10) <= finalDate) {
        if (!cardapio.dateLessDateOfProcess(date.DTCARDIAPME.substring(0, 10))) hasBloqDay = true;
      }
    });
    return hasBloqDay;
  }
}
